/*
 * Copyright (c) 2001 Jilles van Gurp
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package newstate.tools;

import newstate.*;
import newstate.tools.*;
import javax.swing.*;
import java.awt.*;
import java.util.*;

public class FSMController extends JFrame
{
        JTextField state = new JTextField();
        JButton executeButton = new JButton("Dispatch Event");

        JList events = new JList();

        FSMContext fsmc;
        
        public FSMController(FSMContext c)
        {
                fsmc = c;
                // user interface init
                setSize(400,200);
                Container p = getContentPane();
                p.setLayout(new BorderLayout());
                p.add("North", state);
                p.add("Center", new JScrollPane(events));
                JPanel buttons = new JPanel();
                buttons.setLayout(new FlowLayout());
                buttons.add(executeButton);
                p.add("East", buttons);
                executeButton.addActionListener(new AbstractAction() {
                        public void actionPerformed(java.awt.event.ActionEvent e)
                        {
                                String evt = (String)events.getSelectedValue();
                                if(e!=null)
                                {
                                        fsmc.dispatch(evt, null);
                                        updateGUI();
                                }
                        }
                });
                updateGUI();
        }
        
        void updateGUI()
        {
                events.setListData(fsmc.getEvents());
                events.setSelectedIndex(0);
                state.setText("" + fsmc.getState());
        }
        
        public class LogWriter implements FSMAction
        {
                String message;
                
                public LogWriter(String s)
                {
                        setMessage(s);
                }
                
                public void execute(FSMContext fsmc, Object data)
                {
                        JTextArea log = (JTextArea)fsmc.get("log"); 
                        log.append(getMessage() + '\n');                
                }
                
                public void setMessage(String m) { message = m; }
                public String getMessage() { return message; }
        }
}
